import unittest
from ttpnav import MitreData

class TestMitreData(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.mitre_data = MitreData()

    def test_get_technique_by_attack_id(self):
        technique_id = "T1134"
        technique = self.mitre_data.get_technique(technique_id)
        self.assertIsNotNone(technique)
        self.assertIsNotNone(technique.id)
        self.assertIsNotNone(technique.name)
        self.assertIsNotNone(technique.description)

        print(f"External ID: {technique.external_id}")
        print(f"Technique Name: {technique.name}")
        print(f"Technique Description: {technique.description}\n")
        for m in technique.mitigations:
            print(f"External ID: {m.external_id}")
            print(f"Mitigation Keys: {m.__dict__.keys()}\n")
            break
        for d in technique.detections:
            print(f"Detection External ID: {d.external_id}")
            print(f"Detection Keys: {d.__dict__.keys()}\n")
            break
        for pe in technique.procedureExamples:
            print(f"Procedure Example Keys: {pe.__dict__.keys()}\n")
            break
        for g in technique.groups:
            print(f"Group Example Keys: {g.__dict__.keys()}\n")
            break
        for s in technique.softwares:
            print(f"Software Example Keys: {s.__dict__.keys()}\n")
            break

    def test_get_tool(self):
        tool = self.mitre_data.get_tool('S0039')
        self.assertIsNotNone(tool.id)
        self.assertIsNotNone(tool.name)
        self.assertIsNotNone(tool.description)

        print(f"External ID: {tool.external_id}")
        print(f"Tool Name: {tool.name}")
        print(f"Description: {tool.description}\n")
        for group in tool.groups:
            print(group.name)
        for technique in tool.techniques:
            print("technique")
            for pe in technique.procedureExamples:
                
                print(pe.source_attack_id)
                print(pe.description)

    def test_get_tools(self):
        tools = self.mitre_data.get_tools()
        self.assertTrue(len(tools) > 0)
        for tool in tools:
            self.assertIsNotNone(tool.id)
            self.assertIsNotNone(tool.name)
            self.assertIsNotNone(tool.description)

            print(f"External ID: {tool.external_id}")
            print(f"Tool Name: {tool.name}")
            print(f"Description: {tool.description}\n")
            for group in tool.groups:
                print(group.name)
            for technique in tool.techniques:
                print("technique")
                for pe in technique.procedureExamples:
                    
                    print(pe.source_attack_id)
                    print(pe.description)
            break

    def test_get_groups(self):
        groups = self.mitre_data.get_groups()
        self.assertTrue(len(groups) > 0)
        for group in groups:
            self.assertIsNotNone(group.id)
            self.assertIsNotNone(group.name)
            self.assertIsNotNone(group.description)

            print(f"External ID: {group.external_id}")
            print(f"Group Name: {group.name}")
            print(f"Description: {group.description}\n")
            print(f"{group.__dict__.keys()}\n")
            for technique in group.techniques:
                print("technique")
                for pe in technique.procedureExamples:
                    
                    print(pe.source_attack_id)
                    print(pe.description)
            break

    def test_get_tactics(self):
        tactics = self.mitre_data.get_tactics()
        self.assertTrue(len(tactics) > 0)
        for tactic in tactics:
            self.assertIsNotNone(tactic.id)
            self.assertIsNotNone(tactic.name)
            self.assertIsNotNone(tactic.description)

            print(f"External ID: {tactic.external_id}")
            print(f"Tactic Name: {tactic.name}")
            print(f"Description: {tactic.description}\n")
            print(f"{tactic.__dict__.keys()}\n")

    def test_get_techniques(self):
        techniques = self.mitre_data.get_parent_techniques()
        self.assertTrue(len(techniques) > 0)
        for technique in techniques:
            self.assertIsNotNone(technique.id)
            self.assertIsNotNone(technique.name)
            self.assertIsNotNone(technique.description)

            print(f"External ID: {technique.external_id}")
            print(f"Technique Keys: {technique.__dict__.keys()}\n")
            for m in technique.mitigations:
                print(f"External ID: {m.external_id}")
                print(f"Mitigation Keys: {m.__dict__.keys()}\n")
                break
            for d in technique.detections:
                print(f"Detection External ID: {d.external_id}")
                print(f"Detection Keys: {d.__dict__.keys()}\n")
                break
            for pe in technique.procedureExamples:
                print(f"Procedure Example Keys: {pe.__dict__.keys()}\n")
                print(pe.description)
                break
            for g in technique.groups:
                print(f"Group Example Keys: {g.__dict__.keys()}\n")
                break
            for s in technique.softwares:
                print(f"Software Example Keys: {s.__dict__.keys()}\n")
                break
            break


    def test_get_subtechniques(self):
        subtechniques = self.mitre_data.get_subtechniques()
        self.assertTrue(len(subtechniques) > 0)
        for subtechnique in subtechniques:
            self.assertIsNotNone(subtechnique.id)
            self.assertIsNotNone(subtechnique.name)
            self.assertIsNotNone(subtechnique.description)

            print(f"External ID: {subtechnique.external_id}")
            print(f"Subtechnique Keys: {subtechnique.__dict__.keys()}\n")
            print(f"Parent ID: {subtechnique.parent_external_id}")
            for m in subtechnique.mitigations:
                print(f"External ID: {m.external_id}")
                print(f"Mitigation Keys: {m.__dict__.keys()}\n")
                break
            for d in subtechnique.detections:
                print(f"Detection External ID: {d.external_id}")
                print(f"Detection Keys: {d.__dict__.keys()}\n")
                break
            for pe in subtechnique.procedureExamples:
                print(f"Procedure Example Keys: {pe.__dict__.keys()}\n")
                print(pe.description)
                break
            for g in subtechnique.groups:
                print(f"Group Example Keys: {g.__dict__.keys()}\n")
                break
            for s in subtechnique.softwares:
                print(f"Software Example Keys: {s.__dict__.keys()}\n")
                break
            break

if __name__ == '__main__':
    unittest.main()


from mitreattack.stix20 import MitreAttackData
import json
import os
import requests

MITRE_ATTACK_JSON_URL = "https://raw.githubusercontent.com/mitre/cti/master/enterprise-attack/enterprise-attack.json"
MITRE_ATTACK_JSON_PATH = "enterprise-attack.json"

class Tactic:
    def __init__(self, mitre_attack_data, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)
        self.external_id = mitre_attack_data.get_attack_id(self.id)

class Technique:
    def __init__(self, mitre_attack_data, parent_external_id=None, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)
        self.external_id = mitre_attack_data.get_attack_id(self.id)
        self.mitigations = kwargs.get("mitigations", [])
        self.parent_external_id = parent_external_id
        self.detections = kwargs.get("detections", [])
        self.procedureExamples = kwargs.get("procedureExamples", [])
        self.groups = kwargs.get("groups", [])
        self.softwares = kwargs.get("softwares", [])

class Mitigation:
    def __init__(self, mitre_attack_data, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)
        self.external_id = mitre_attack_data.get_attack_id(self.id)

class Detection:
    def __init__(self, mitre_attack_data, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)
        datasource = mitre_attack_data.get_object_by_stix_id(self.x_mitre_data_source_ref)
        self.external_id = mitre_attack_data.get_attack_id(datasource.id)

class ProcedureExample:
    def __init__(self, mitre_attack_data, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)
        source_object = mitre_attack_data.get_object_by_stix_id(self.source_ref)
        attack_id = mitre_attack_data.get_attack_id(source_object.id)
        self.source_attack_id = attack_id

class Group:
    def __init__(self, mitre_attack_data, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)
        self.external_id = mitre_attack_data.get_attack_id(self.id)

class GroupDetail:
    def __init__(self, mitre_attack_data, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)
        self.external_id = mitre_attack_data.get_attack_id(self.id)
        self.techniques = kwargs.get("techniques", [])
        self.softwares = kwargs.get("softwares", [])
        self.techniques = self.filter_technique_procedure_examples(kwargs.get("techniques", []))

    def filter_technique_procedure_examples(self, techniques):
        for technique in techniques:
            technique.procedureExamples = [example for example in technique.procedureExamples if example.source_attack_id == self.external_id]
        return techniques
    
class SoftwareDetail:
    def __init__(self, mitre_attack_data, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)
        self.external_id = mitre_attack_data.get_attack_id(self.id)
        self.techniques = kwargs.get("techniques", [])
        self.groups = kwargs.get("groups", [])
        self.techniques = self.filter_technique_procedure_examples(kwargs.get("techniques", []))

    def filter_technique_procedure_examples(self, techniques):
        for technique in techniques:
            technique.procedureExamples = [example for example in technique.procedureExamples if example.source_attack_id == self.external_id]
        return techniques

class Software:
    def __init__(self, mitre_attack_data, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)
        self.external_id = mitre_attack_data.get_attack_id(self.id)

class MitreData:
    def __init__(self):
        self.file_path = self.download_mitre_attack_data(MITRE_ATTACK_JSON_PATH)
        self.mitre_attack_data = MitreAttackData(self.file_path)

    @staticmethod
    def download_mitre_attack_data(file_path):
        if not os.path.exists(file_path):
            print("Downloading MITRE ATT&CK data...")
            response = requests.get(MITRE_ATTACK_JSON_URL)
            with open(file_path, 'wb') as file:
                file.write(response.content)
            print("Download complete.")
        return file_path

    def get_groups(self):
        groups = self.mitre_attack_data.get_groups(remove_revoked_deprecated=True)
        group_objects = []
        for group in groups:
            techniques = self.get_techniques_used_by_group(group.id)
            softwares = self.get_software_used_by_group(group.id)
            group_obj = GroupDetail(self.mitre_attack_data, techniques=techniques, softwares=softwares, **group)
            group_objects.append(group_obj)
        return group_objects
    
    def get_group(self, attack_id):
        group = self.mitre_attack_data.get_object_by_attack_id(attack_id, "intrusion-set")
        if group:
            techniques = self.get_techniques_used_by_group(group.id)
            softwares = self.get_software_used_by_group(group.id)
            group_obj = GroupDetail(self.mitre_attack_data, techniques=techniques, softwares=softwares, **group)
            return group_obj
        return None
    
    def get_technique(self, attack_id):
        technique = self.mitre_attack_data.get_object_by_attack_id(attack_id, "attack-pattern")
        if technique:
            mitigations = self.get_mitigations_from_technique(technique.id)
            detections = self.get_technique_detections(technique.id)
            procedureExamples = self.get_procedure_examples_from_technique(technique.id)
            groups = self.get_groups_using_technique(technique.id)
            softwares = self.get_software_using_technique(technique.id)
            technique_obj = Technique(self.mitre_attack_data, mitigations=mitigations, detections=detections, procedureExamples=procedureExamples, groups=groups, softwares=softwares, **technique)
            return technique_obj
        return None

    def get_tactics(self):
        tactics = self.mitre_attack_data.get_tactics(remove_revoked_deprecated=True)
        return [Tactic(self.mitre_attack_data, **tactic) for tactic in tactics]
    
    def get_tools(self):
        softwares = self.mitre_attack_data.get_software(remove_revoked_deprecated=True)
        tool_objects = []
        for tool in softwares:
            techniques = self.get_techniques_used_by_software(tool.id)
            groups = self.get_groups_using_software(tool.id)
            tool_obj = SoftwareDetail(self.mitre_attack_data, techniques=techniques, groups=groups, **tool)
            tool_objects.append(tool_obj)
        return tool_objects
    
    def get_tool(self, attack_id):
        tool = self.mitre_attack_data.get_object_by_attack_id(attack_id, "tool")
        if tool:
            techniques = self.get_techniques_used_by_software(tool.id)
            groups = self.get_groups_using_software(tool.id)
            tool_obj = SoftwareDetail(self.mitre_attack_data, techniques=techniques, groups=groups, **tool)
            return tool_obj
        return None

    def get_groups_using_software(self, software_id):
        groups = self.mitre_attack_data.get_groups_using_software(software_id)
        return [Group(self.mitre_attack_data, **group["object"]) for group in groups]

    def get_parent_techniques(self):
        techniques = self.mitre_attack_data.get_techniques(include_subtechniques=False, remove_revoked_deprecated=True)
        technique_objects = []
        for technique in techniques:
            mitigations = self.get_mitigations_from_technique(technique.id)
            detections = self.get_technique_detections(technique.id)
            procedureExamples = self.get_procedure_examples_from_technique(technique.id)
            groups = self.get_groups_using_technique(technique.id)
            softwares = self.get_software_using_technique(technique.id)
            technique_obj = Technique(self.mitre_attack_data, mitigations=mitigations, detections=detections, procedureExamples=procedureExamples, groups=groups, softwares=softwares, **technique)
            technique_objects.append(technique_obj)
        return technique_objects
    
    def get_subtechniques(self):
        subtechniques = self.mitre_attack_data.get_subtechniques(remove_revoked_deprecated=True)
        subtechnique_objects = []
        for subtechnique in subtechniques:
            mitigations = self.get_mitigations_from_technique(subtechnique.id)
            detections = self.get_technique_detections(subtechnique.id)
            procedureExamples = self.get_procedure_examples_from_technique(subtechnique.id)
            groups = self.get_groups_using_technique(subtechnique.id)
            softwares = self.get_software_using_technique(subtechnique.id)

            parent_technique = self.mitre_attack_data.get_parent_technique_of_subtechnique(subtechnique.id)
            parent_external_id = self.mitre_attack_data.get_attack_id(parent_technique[0]["object"].id) if parent_technique else None

            subtechnique_obj = Technique(self.mitre_attack_data, mitigations=mitigations, detections=detections, procedureExamples=procedureExamples, groups=groups, softwares=softwares, parent_external_id=parent_external_id, **subtechnique)
            subtechnique_objects.append(subtechnique_obj)
        return subtechnique_objects
    
    def get_techniques_used_by_software(self, software_id):
        techniques = self.mitre_attack_data.get_techniques_used_by_software(software_id)
        technique_objects = []
        for t in techniques:
            technique = t["object"]
            mitigations = self.get_mitigations_from_technique(technique.id)
            detections = self.get_technique_detections(technique.id)
            procedureExamples = self.get_procedure_examples_from_technique(technique.id)
            technique_obj = Technique(self.mitre_attack_data, mitigations=mitigations, detections=detections, procedureExamples=procedureExamples, **technique)
            technique_objects.append(technique_obj)
        return technique_objects
    
    def get_mitigations_from_technique(self, technique_id):
        mitigations = self.mitre_attack_data.get_mitigations_mitigating_technique(technique_id)
        return [Mitigation(self.mitre_attack_data, **mitigation["object"]) for mitigation in mitigations]
    
    def get_technique_detections(self, technique_id):
        detections = self.mitre_attack_data.get_datacomponents_detecting_technique(technique_id)
        return [Detection(self.mitre_attack_data, **detection["object"]) for detection in detections]
    
    def get_procedure_examples_from_technique(self, technique_id):
        procedure_examples = self.mitre_attack_data.get_procedure_examples_by_technique(technique_id)
        return [ProcedureExample(self.mitre_attack_data, **procedure_example) for procedure_example in procedure_examples]
    
    def get_groups_using_technique(self, technique_id):
        groups = self.mitre_attack_data.get_groups_using_technique(technique_id)
        return [Group(self.mitre_attack_data, **group["object"]) for group in groups]
    
    def get_software_using_technique(self, technique_id):
        softwares = self.mitre_attack_data.get_software_using_technique(technique_id)
        return [Software(self.mitre_attack_data, **software["object"]) for software in softwares]
    
    def get_software_used_by_group(self, group_id):
        softwares = self.mitre_attack_data.get_software_used_by_group(group_id)
        return [Software(self.mitre_attack_data, **software["object"]) for software in softwares]
    
    def get_techniques_used_by_group(self, group_id):
        techniques = self.mitre_attack_data.get_techniques_used_by_group(group_id)
        technique_objects = []
        for t in techniques:
            technique = t["object"]
            mitigations = self.get_mitigations_from_technique(technique.id)
            detections = self.get_technique_detections(technique.id)
            procedureExamples = self.get_procedure_examples_from_technique(technique.id)
            technique_obj = Technique(self.mitre_attack_data, mitigations=mitigations, detections=detections, procedureExamples=procedureExamples, **technique)
            technique_objects.append(technique_obj)
        return technique_objects
    


from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="ttpnav",
    version="0.1.1",
    author="Joseph Fisher",
    author_email="jfisher@cyntel.com",
    description="A package to navigate MITRE ATT&CK data easily.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/fish-not-phish/ttpnav",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'mitreattack-python'
    ],
)
